#!/usr/bin/env bash

# common.sh - common functions.
# Copyright (c) 2022 ge <https://nixhacks.net/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

log() {
    # Logger. Write message to __log_file.
    #
    # Usage: log [-p] MESSAGE
    #
    # Can read from STDIN. Removes ANSI escape codes from log message.
    # Echo message to STDOUT with '-p' option.

    local message
    local print
    local log_file

    log_file="$__log_file"

    # Read message from STDIN if args not passed.
    [[ ! -t 0 ]] || [[ "$#" == 0 ]] && message="$(cat <&0)"

    # Set log date format
    [ "$log_date_fmt" ] || log_date_fmt="$__log_date_fmt"

    while (( "$#" )); do
        case "$1" in
            -p) print=1; shift;;
            -)  message="$(cat <&0)"; shift "$#";;
            *)  message="$*"; shift "$#";;
        esac
    done

    [[ "$print" == 1 ]] && echo -e "$message"

    while read -r line; do
        if [[ "$line" ]]; then
            printf '[%s] [%s] %s\n' "$(date +"$log_date_fmt")" "$0" "$line" >> "$log_file"
        fi
    done <<< "$(sed -r 's/\x1B\[(([0-9]+)(;[0-9]+)*)?[m,K,H,f,J]//g' <<< "$message")"
}

err() {
    # Handle error. Print errors to STDERR, log, process
    # and exit from script.
    #
    # Usage: err [-eao] MESSAGE

    local message
    local errexit
    local errappend
    local onerr

    [[ ! -t 0 ]] || [[ "$#" == 0 ]] && message="$(cat <&0)"

    # Split combined short options, e.g. '-abc' to '-a' '-b' '-c'
    for args in "$@"; do
        shift
        case "$args" in
            -*)
                args="$(echo "${args:1}" | grep -o . | xargs -I {} echo -n '-{} ')"
                set -- "$@" $args;;
            *)
                set -- "$@" "$args";;  # save positional arguments
        esac
    done

    while (( "$#" )); do
        case "$1" in
            -e) errexit=1; shift;;
            -a) errappend=1; shift;;
            -o) onerr=1; shift;;
            -)  message="$(cat <&0)"; shift "$#";;
            *)  message="$*"; shift "$#";;
        esac
    done

    log "$message"
    echo -e "$message" >&2
    [ "$errappend" ] && errors+=("$message")

    # Run user defined function on_error()
    if [ "$onerr" ]; then
        if is_function_set on_error; then
            on_error
        fi
    fi

    # Exit
    if [ "$errexit" ]; then
        log "Backup ERROR: Exiting with previous errors"; exit 1
    fi
}

try() {
    # Commands wrapper for error handling
    #
    # Usage: try COMMAND

    if eval "$@" 2>> "$__log_file"; then
        return 0 # Success
    else
        err -a "Error: Something went wrong when executing command:"
        err -a "    $*"
        err -aeo "Check $__log_file log for details."
    fi
}
